use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007006
use Test::Spelling 0.17;
use Pod::Wordlist;

set_spell_cmd('aspell list');
add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
ABYTES
ASE
Alex
BEFORENMBYTES
Bacause
Burzyński
BytesLocker
CPAN
ChaCha
Chase
Crypt
Data
Diffie
GCM
Ghz
Heartbleed
IPsec
KEYBYTES
Legge
MACBYTES
Murmurhash
NIST
NONCEBYTES
NPUBBYTES
NaCl
OpenSSL
PRF
PUBLICKEYBYTES
SALTBYTES
SCALARBYTES
SECRETKEYBYTES
SEEDBYTES
STRBYTES
SipHash
Sodium
TLS
Timothy
Whitener
aead
ajgb
auth
beforenm
box
capoeirab
chacha
cryptographic
decrypt
decrypted
decrypts
eg
generichash
hash
hmacsha
kb
keygen
keypair
keyspace
lib
libsodium
memzero
nonces
onetimeauth
packageable
plaintext
precalculated
precalculation
pseudorandom
pwhash
repudiability
scalarmult
scrypt
secretbox
sha
shorthash
sign
str
stream
subkey
timlegge
unforgeability
verifiability
xsalsa
yescrypt
